AJS.$(document).ready( function(){	

	console.log('LDAP query.js loaded');
	
	function toCsv(anAtt, valueMap) {
		var result='';
		for( var aValue in valueMap[anAtt]) {
			result=result+valueMap[anAtt][aValue]+', ';
		}
		
		if (result.charAt( result.length-2 ) == ",") {
		    result = result.slice(0, -2)
		}
				
		return result;
	}
	
	function htmlEncode(value){
  		return AJS.$('<div/>').text(value).html();
	}

	function addAttributeRow(anAttIdx, attributes, valueMap) {
		var anAtt=attributes[anAttIdx];
		var newRow='\n\t\t<tr><th style="width: 100px;">'+anAtt+'</th><td>';
		if (valueMap[anAtt].length>1) {
			newRow+='<ul>';
			for( var aValueIdx in valueMap[anAtt]) {
				newRow+='<li>'+htmlEncode(valueMap[anAtt][aValueIdx])+'</li>';
			}
			newRow+='</ul>';
		} else {
			newRow+=htmlEncode(valueMap[anAtt][0]);
		}
		newRow+='</td></tr>';
 	    return newRow;	
	}
	
	function isUser(valueMap) {
		var isUser=false;
		var classes=valueMap['objectClass'];
		if (classes) {
			for( var i in classes) {
				if (classes[i]=='user') {
					isUser=true;
					break;
				}
			}
		}
		return isUser;
	}
	
	function isGroup(valueMap) {
		
		var isGroup=false;
		var classes=valueMap['objectClass'];
		if (classes) {
			for( var i in classes) {
				if (classes[i]=='group') {
					isGroup=true;
					break;
				}
			}
		}
		var cnAttr=valueMap['cn'];
		console.log("cn: "+cnAttr[0]+" : is a group: "+isGroup);
		return isGroup;
	}
	
	function renderSummary(resultIndex, attributes, valueMap)
	{	
		var typeIndicator='';
		if (isUser(valueMap)) {
			typeIndicator='<span class="aui-icon aui-icon-small aui-iconfont-user" title="User"></span>';
		} else if (isGroup(valueMap)) {
			typeIndicator='<span class="aui-icon aui-icon-small aui-iconfont-group" title="Group"></span>';
		}

		var summary=typeIndicator+' '+resultIndex;	

	 	for( var anAttIdx in attributes) {
	 		var anAtt=attributes[anAttIdx];
	 		if (anAtt=='cn') {	 			
	 	    	summary+=', cn='+htmlEncode(valueMap[anAtt][0]);
	 	    	break;
	 	    }	 	    
	 	}
		
	 	for( var anAttIdx in attributes) {
	 		var anAtt=attributes[anAttIdx];
	 		if (anAtt=='sAMAccountName') {	 			
	 	    	summary+=', sAMAccountName='+htmlEncode(valueMap[anAtt][0]);
	 	    	break;
	 	    }	 	    
	 	}
	 	
	 	for( var anAttIdx in attributes) {
	 		var anAtt=attributes[anAttIdx];
	 		if (anAtt=='mail') {	 			
	 	    	summary+=', mail='+htmlEncode(valueMap[anAtt][0]);
	 	    	break;
	 	    }
		}	 	
	 			
		return summary;
	}
	
	function renderActionList(valueMap) {
		var listHtml='';
		if (isGroup(valueMap) && valueMap['mail']) {
			var mailAttr=valueMap['mail'];
			var mailAddress=mailAttr[0];
			listHtml+='&nbsp;<A HREF="#"><span data-email="'+mailAddress
				+'" data-configId="'+AJS.$("#ldapFilterConfig option:selected").val()
				+'" data-expand="'+AJS.$("#expandAttributes").val()
				+'" title="Export registered JIRA users within groups and sub-groups" class="aui-icon aui-icon-small aui-iconfont-bp-jira jemh-ldap-user-export"></span></A>';
		}
		return listHtml;
	}
	
	function renderExpandableResult(resultIndex, attributes, valueMap, subResults) {
		var expandableResult='';
		expandableResult+='	<div class="module jemh-toggle-wrap toggle-wrap collapsed" id="jemh-result'+resultIndex+'-module" width="100%">\n';
		expandableResult+='		<div id="result'+resultIndex+'-module_heading" class="mod-header">\n';
		expandableResult+='       	<svg xmlns="http://www.w3.org/2000/svg" width="14" height="14"><path fill="#344563" fill-rule="evenodd" d="M3.3 4.8a1 1 0 0 0 0 1.4l3 3c.1.2.4.3.7.3s.6-.1.8-.3l3-3c.3-.4.3-1 0-1.4a1 1 0 0 0-1.5 0L7 7.1 4.7 4.8a1 1 0 0 0-1.4 0z"/></svg>';
		expandableResult+='			<h3 class="toggle-title">'+renderSummary(resultIndex, attributes, valueMap)+'</h3>\n';
		expandableResult+='		  	&nbsp;<strong>actions:</strong>\n';
		expandableResult+='         '+renderActionList(valueMap);
		expandableResult+='		</div>\n';
		expandableResult+='		<div class="mod-content">\n';
		expandableResult+='		<table class="aui table-row-hover">\n';
	 	
	 	for( var anAttIdx in attributes) { expandableResult+= addAttributeRow(anAttIdx, attributes, valueMap); }
		
  		if (subResults && subResults.length>0) {
  			expandableResult+='       <tr>';
			expandableResult+='         <th>Sub Results</th>\n';
			expandableResult+='         <td>\n';
			expandableResult+='		      <table class="aui table-row-hover">\n';
			expandableResult+='             <tr>\n';
			expandableResult+='               <td style="width: 100%;">\n';
			expandableResult+='               <p>There are '+subResults.length+' expanded users/groups:</p>\n';
			
	  		for( var subResultIdx in subResults ) {
	  		  console.log('adding record row '+resultIndex+' sub-result '+(+subResultIdx +1)+' of '+subResults.length);

			  var subResult=subResults[subResultIdx];
	  		  var atts=subResult.attributes;
	  		  var values=subResult.values;
	  		  var nextLevelSubResults=subResult.subResults;
		  	  var renderId = (+subResultIdx + +1);
		  	  expandableResult+=renderExpandableResult(renderId, atts, values, nextLevelSubResults);
		  	}

		  	expandableResult+='             </td>\n';
		  	expandableResult+='           </tr>\n';			
			expandableResult+='		    </table>\n';
			expandableResult+='       </td>\n';
			expandableResult+='       </tr>\n';
	  	}

		expandableResult+='		</table>\n';
		expandableResult+='		</div>\n';
		expandableResult+='	</div>\n';

		return expandableResult;		
	}
	
	function handlePost() {
	
		AJS.$("#queryFailureDiv").empty();
		AJS.$("#queryResultsDiv").html("<p>Processing...</p>");
	
		var postData=AJS.$('#jemhLdapQueryForm').serialize();
	  	var formUrl = AJS.$('#jemhLdapQueryForm').attr('action');
	  	
		//contentType: sent to the server as a header
		//dataType is the response
		var jqXHR=AJS.$.ajax({
			type:"POST",
			url:formUrl,			
			dataType:"text",
			data:postData,
			timeout:60000
		}); //post
				
	  	jqXHR.done(function(data, textStatus, errorThrown) {  			  		
	  		var jsonObj=JSON.parse(data);

			if (jsonObj.error != "")
			{
				console.log("error: "+jsonObj.error);
			}
			
			AJS.$('#queryResultsDiv').html('<p>There were '+jsonObj.results.length+' top-level records returned in '+jsonObj.queryTime+' ms:</p>');
			
	  		//console.log("Post returned with: "+jsonObj.results.length+" results");
	  		
	  		for( var resultIdx in jsonObj.results) {
		  		var record=jsonObj.results[resultIdx];
		  		var atts=record.attributes;
		  		var values=record.values;
		  		var subResults=record.subResults;
			  	
		  		var resultIndex = (+resultIdx + +1);
		  		
		  		console.log('adding record row ' +resultIndex);
		  		var rowData=renderExpandableResult(resultIndex, atts, values, subResults);
		  		AJS.$('#queryResultsDiv').append(rowData);
		  		AJS.$('.jemh-ldap-user-export').click(function() {
		  			var groupAddress=AJS.$( this ).attr('data-email');
		  			var ldapConfigId=AJS.$( this ).attr('data-configId');
		  			var expandAttributes=AJS.$( this ).attr('data-expand');
		  			var atl_token = AJS.$('#atl_token').val();
		  			console.log('click: '+ groupAddress);
		  			
		  			var pathToResource=AJS.params.baseURL+'/rest/jemh/latest/ldap/users?groupAddress='+groupAddress+'&ldapConfigId='+ldapConfigId+'&expandAttributes='+expandAttributes+'&atl_token='+atl_token;
		  			window.open(pathToResource, '_blank');
		  		});
		  	}
	  			
	  	}).fail(function(data, textStatus, errorMessage) {
			//console.log("RUN fail: textStatus="+textStatus+", errorMessage: "+errorMessage);
			AJS.$('#queryResultsDiv').empty();
			AJS.messages.generic("#queryFailureDiv", {
			   title:"LDAP Query Post failure",
			   body: '<p>Query failed, status: '+textStatus+', error: '+errorMessage+'</p>'
			});			
	  	});
	}
	
	function validateFilter(filter)
	{		
		var bracketsMatch = (filter.split('(').length == filter.split(')').length);
		if (bracketsMatch == false) {
			//console.log("filter failed bracket check: "+filter);
			AJS.$("#queryFailureDiv").empty();
			AJS.messages.generic("#queryFailureDiv", {
			   title:"Bracket mismatch",
			   body: '<p>Filters must have matched open "(" and close ")" brackets, eg <pre>(&(mail=*)(sAMAccountName=andy))</pre></p>'
			});			
		}
		
		return bracketsMatch;
	}

	function runQueryIfValid() {
		var ldapConfigId=AJS.$("#ldapFilterConfig option:selected").val();
		var filter=AJS.$("#ldapFilter").val().trim();
		var maxResults=AJS.$("#maxResults").val().trim();
		if ( ldapConfigId!=0 && maxResults.length>0 && filter.length>0 && validateFilter(filter) )
		{
			//console.log('filter is valid');
			handlePost();
		}
	}

	function resetSelect() {
	
		var ldapConfigId=AJS.$("#ldapFilterConfig option:selected").val();
		//console.log('ldapConfigId: '+ldapConfigId);
		if (ldapConfigId != 0 )
		{
			//console.log('LDAP config selected');
			AJS.$("#ldapFilter").removeAttr('disabled');
			AJS.$('#queryResultsDiv').empty();
			var filter=AJS.$("#ldapFilter").val().trim();
			if (filter.length==0) {
				AJS.$("#ldapFilter").val("(sAMAccountName=*)");
			}
			runQueryIfValid();
		}
		else
		{	
			//console.log('LDAP config cleared');
			AJS.$("#ldapFilter").attr('disabled','disabled');
			AJS.$('#queryResultsDiv').html('<p>Select an LDAP configuration in order to enter an LDAP filter</p>');
		}
	}
	
	resetSelect();

	AJS.$("#ldapQueryRefresh").click(function() {
		runQueryIfValid();
	});

	var queryTimeout = null;
	AJS.$(".ldap-query-field").on("input", function() {
		clearTimeout(queryTimeout);
		queryTimeout = setTimeout(runQueryIfValid, 250);
	});

	AJS.$("#ldapFilterConfig").change(function() {
		resetSelect();
		runQueryIfValid();
	});
});
