AJS.$(document).ready(function () {

    var baseUrl = window.location.protocol + "//" + window.location.hostname + (window.location.port ? ':' + window.location.port : '') + AJS.contextPath();
    var projectMappingId = AJS.$('#projConfigId').val();
    var profileId = AJS.$("#jemhProfileIdHolder").val();
    var atl_token = AJS.$('#atlassian-token').attr('content');


    // https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/encodeURIComponent
    // here is an alternative to the above function
    function encodeRFC5987ValueChars2(str) {
        return encodeURIComponent(str).
            // Note that although RFC3986 reserves "!", RFC5987 does not,
            // so we do not need to escape it
            replace(/['()*]/g, c => "%" + c.charCodeAt(0).toString(16)). // i.e., %27 %28 %29 %2A
            // The following are not required for percent-encoding per RFC5987,
            // so we can allow for a little better readability over the wire: |`^
            replace(/%(7C|60|5E)/g, (str, hex) => String.fromCharCode(parseInt(hex, 16)));
    }

    function apply() {
        var testCaseId = AJS.$("#dynEvalTestCaseSelect").val();

        if (testCaseId == "") {
            return;
        }

        //console.log('running with profileId '+profileId+' and testCaseId '+testCaseId);

        //need to now build a payload containing all expressions in their edited state, and get the result output for processing
        // http://localhost:8080/rest/jemh/latest/pmdelimregexp/process/2

        var targetUrl = baseUrl + '/rest/jemh/latest/pmdelimregexp/process/' + profileId + '/' + projectMappingId + '/' + testCaseId + '?atl_token=' + atl_token;
        var formObj = AJS.$('<form>');

        //MODE
        formObj.append("<input type='hidden' name='dynEvalModeSelect' value='" + AJS.$('#dynEvalModeSelect').val() + "'>");

        AJS.$(".jemhSubjectCleanupRegexp").each(function (i, obj) {
            var regexpVal = AJS.$(obj).val();
            if (regexpVal.trim() != "") {
                //console.log('found subject cleanup regexp #'+i+', value is : '+regexpVal);
                var escaped = AJS.$('<div></div>').html(regexpVal).text();
                formObj.append('<input type="text" name="cleanupRegexps" value="subject_' + escaped + '">');
            } else {
                //console.log('ignoring empty jemhCleanupRegexp');
            }
        });

        AJS.$(".jemhBodyCleanupRegexp").each(function (i, obj) {
            var regexpVal = AJS.$(obj).val();
            if (regexpVal.trim() != "") {
                //console.log('found body cleanup regexp #'+i+', value is : '+regexpVal);
                var escaped = AJS.$('<div></div>').html(regexpVal).text();
                formObj.append('<input type="text" name="cleanupRegexps" value="body_' + escaped + '">');
            } else {
                //console.log('ignoring empty jemhCleanupRegexp');
            }
        });

        AJS.$(".jemhDelimiterRegexp").each(function (i, obj) {
            var regexpVal = AJS.$(obj).val();
            if (regexpVal.trim() != "") {
                //console.log('found body delim regexp #'+i+', value is : '+regexpVal);

                var id = AJS.$(obj).closest('tr').attr('id').split('_')[1];
                var isCreate = AJS.$('#on_create_' + id).attr('checked') == 'checked';
                var isComment = AJS.$('#on_comment_' + id).attr('checked') == 'checked';

                //console.log('create='+isCreate+',comment='+isComment);

                var escaped = AJS.$('<div></div>').html(regexpVal).text();
                formObj.append('<input type="text" name="delimiterRegexps" value="' + isCreate + '_' + isComment + '_' + escaped + '">');
            } else {
                //console.log('ignoring empty jemhDelimiterRegexp');
            }
        });

        var postData = formObj.serialize();
        console.log('postData: ' + postData);

        AJS.$('#dynEvalRawDivOutput').html('<p>updating...</p>');
        AJS.$('#dynEvalModifiedDivOutput').html('<p>updating...</p>');
        AJS.$('#dynEvalRawSubDivOutput').html('<p>updating...</p>');
        AJS.$('#dynEvalModifiedSubDivOutput').html('<p>updating...</p>');

        //contentType: sent to the server as a header
        //dataType is the response
        var jqXHR = AJS.$.ajax({
            type: "POST",
            contentType: "application/x-www-form-urlencoded; charset=utf-8",
            url: targetUrl,
            dataType: "json",
            data: postData,
            timeout: 60000
        }); //post

        jqXHR.done(function (data, textStatus, errorThrown) {
            //console.log('done '+data);

            if (data.error) {
                console.log('Exception: ' + data.error);
            } else {
                //console.log('original subject: '+data.rawSubject);
                //console.log('modifiedContent: '+data.modifiedContent);
                //console.log('modifiedSubject: '+data.modifiedSubject);

                //
                // Apply Subject
                //
                var escapedRawSubject = $("<div>").text(data.rawSubject).html();
                AJS.$('#dynEvalRawSubDivOutput').html(escapedRawSubject);

                var escapedModifiedSubject = $("<div>").text(data.modifiedSubject).html();
                AJS.$('#dynEvalModifiedSubDivOutput').html(escapedModifiedSubject);

                AJS.tabs.change(jQuery('a[href="#tabs-pmregexpsub-eval-first"]'));

                //
                // Apply Content Content
                //
                var escapedRawContent = $("<div>").text(data.rawContent).html();
                AJS.$('#dynEvalRawDivOutput').html(escapedRawContent);

                var escapedModifiedContent = $("<div>").text(data.modifiedContent).html();
                AJS.$('#dynEvalModifiedDivOutput').html(escapedModifiedContent);

                // Apply Logs

                //
                // SUMMARY
                //
                try {
                    //clear subject
                    AJS.$('#dynEvalProcSummaryExecutionDivOutput').html('');

                    if (data.subjectCleanupLog) {
                        var html = '';
                        //console.log('subject cleanup msgs : '+data.subjectCleanupLog.length);
                        var obj = data.subjectCleanupLog;
                        for (var i = 0, len = obj.length; i < len; i++) {
                            //console.log('subjectCleanup #'+i+' : '+obj[i].message);
                            html = html + AJS.$("<div>").text(obj[i].message).html() + "\n";
                        }
                        AJS.$('#dynEvalProcSummaryExecutionDivOutput').html(html);
                    }
                } catch (e) {
                    console.log('cannot get subject cleanup log');
                }

                //
                // BODY
                //
                var html = '';
                try {
                    if (data.bodyCleanupLog) {
                        //console.log('body cleanup msgs : '+data.bodyCleanupLog.length);
                        obj = data.bodyCleanupLog;
                        for (var i = 0, len = obj.length; i < len; i++) {
                            //console.log('bodyCleanup #'+i+' : '+obj[i].message);
                            html = html + AJS.$("<div>").text(obj[i].message).html() + "\n";
                        }
                    }
                } catch (e) {
                    console.log('cannot get body cleanup log');
                }

                try {
                    if (data.bodyDelimLog) {
                        //console.log('body delim msgs : '+data.bodyDelimLog.length);
                        obj = data.bodyDelimLog;
                        for (var i = 0, len = obj.length; i < len; i++) {
                            //console.log('bodyDelim #'+i+' : '+obj[i].message);
                            html = html + AJS.$("<div>").text(obj[i].message).html() + "\n";
                        }
                    }
                } catch (e) {
                    console.log('cannot get body delim log');
                }
                AJS.$('#dynEvalProcBodyExecutionDivOutput').html(html);


                AJS.tabs.change(jQuery('a[href="#tabs-pmregexp-eval-first"]'));
            }
        });
    }

    //
    //  DELIMETER
    //

    function hookAddRegexpDelimItem() {
        AJS.$("#newbodydelimregexp_button").click(function (e) {
            e.preventDefault();
            // console.log('add delimeter');

            var createUrl = baseUrl + '/rest/jemh/latest/pmdelimregexp/delim/' + projectMappingId + '?atl_token=' + atl_token;
            //console.log('createRegexp url: '+createUrl);


            var id = 'unsaved-' + Date.now();
            var regexp = 'an-example-regexp-changeme';
            //console.log('need to add row, entityId: '+id+', useOnComment: '+useOnCreate+', useOnComment='+useOnComment+', regexp='+regexp);

            //addrow
            var markup = "<tr id='bodydelimregexp_" + id + "'>"
                + "<td>" + id + "</td>"
                + "<td><div><input class='checkbox jemhBodyDelimCheckbox' type='checkbox' name='on_create_" + id + "' id='on_create_" + id + "' checked='checked' ></div></td>"
                + "<td><div><input class='checkbox jemhBodyDelimCheckbox' type='checkbox' name='on_comment_" + id + "' id='on_comment_" + id + "' checked='checked' ></div></td>"
                + "<td><input class='text long-field jemhDelimiterRegexp jemhAutoEvalFocusMember' type='text' name='regexp_" + id + "' id='regexp_" + id + "' style='max-width: 100%;' value='" + regexp + "'></td>"
                + "<td>"
                + "<a class='deleteDelimRegexpLink' href='" + id + "'><span id='" + id + "' title='Delete' class='aui-icon aui-icon-small aui-iconfont-delete'></span></a>"
                + "</td>";

            AJS.$('#regexpBodyDelimTableBody').append(AJS.$(markup).hide().show('fast'));

            //hook delete listener
            AJS.$('a[href=' + id + ']').click(function (e) {
                //console.log('newly added item delete');
                e.preventDefault();
                deleteDelimRegexp(AJS.$(this));
                return false;
            });

            //hook blur listener
            AJS.$('input[name=regexp_' + id + ']').blur(function () {
                //console.log('blur');
                apply();
            });
	
            AJS.$('input[name=on_create_' + id + ']').change(function (e) {
                apply();
            });
            
            AJS.$('input[name=on_comment_' + id + ']').change(function (e) {
                apply();
            });

            return false;
        });
    }

    //
    // CLEANUP
    //
    function hookAddSubjectCleanupItem() {
        AJS.$("#newsubjectcleanupregexp_button").click(function (e) {
            e.preventDefault();
            //console.log('add subject cleanup clicked');

            var id = 'unsaved-' + Date.now();
            var regexp = 'default-subject-cleanup-changme';
            var type = 'subject';
            //console.log('need to add row, entityId: '+id+', type='+type+', regexp='+regexp);

            //addrow
            var markup = "<tr id='subjectcleanupregexp_" + id + "'>"
                + "<td>" + id + "</td>"
                + "<td class='.on-comment'>" + type + "</td>"
                + "<td><input class='text long-field jemhSubjectCleanupRegexp jemhAutoEvalFocusMember' type='text' name='regexp_" + id + "' id='regexp_" + id + "' style='max-width: 100%;' value='" + regexp + "'></td>"
                + "<td>"
                + "<a class='deleteCleanupRegexpLink' href='" + id + "'><span id='" + id + "' title='Delete' class='aui-icon aui-icon-small aui-iconfont-delete'></span></a>"
                + "</td>";

            AJS.$('#regexpSubjectCleanupTableBody').append(AJS.$(markup).hide().show('fast'));

            //hook delete listener
            AJS.$('a[href=' + id + ']').click(function (e) {
                //console.log('newly added item delete');
                e.preventDefault();
                deleteSubjectCleanupRegexp(AJS.$(this));
                return false;
            });

            //hook blur listener
            AJS.$('input[name=regexp_' + id + ']').blur(function () {
                //console.log('blur');
                apply();
            });

            return false;
        });
    }

    function hookAddBodyCleanupItem() {
        AJS.$("#newbodycleanupregexp_button").click(function (e) {
            e.preventDefault();
            //console.log('add body cleanup clicked');

            var id = 'unsaved-' + Date.now();
            var regexp = 'default-body-cleanup-changme';
            var type = 'body';
            //console.log('need to add row, entityId: '+id+', type='+type+', regexp='+regexp);

            //addrow
            var markup = "<tr id='bodycleanupregexp_" + id + "'>"
                + "<td>" + id + "</td>"
                + "<td class='.on-comment'>" + type + "</td>"
                + "<td><input class='text long-field jemhBodyCleanupRegexp jemhAutoEvalFocusMember' type='text' name='regexp_" + id + "' id='regexp_" + id + "' style='max-width: 100%;' value='" + regexp + "'></td>"
                + "<td>"
                + "<a class='deleteCleanupRegexpLink' href='" + id + "'><span id='" + id + "' title='Delete' class='aui-icon aui-icon-small aui-iconfont-delete'></span></a>"
                + "</td>";

            AJS.$('#regexpBodyCleanupTableBody').append(AJS.$(markup).hide().show('fast'));

            //hook delete listener
            AJS.$('a[href=' + id + ']').click(function (e) {
                //console.log('newly added item delete');
                e.preventDefault();
                deleteBodyCleanupRegexp(AJS.$(this));
                return false;
            });

            //hook blur listener
            AJS.$('input[name=regexp_' + id + ']').blur(function () {
                //console.log('blur');
                apply();
            });

            return false;
        });
    }

    function deleteSubjectCleanupRegexp(item) {
        var regexpId = AJS.$(item).attr('href');
        //console.log('deleteSubjectCleanupRegexp() : '+item);
        AJS.$('tr[id=subjectcleanupregexp_' + regexpId + ']').hide('fast', function () {
            AJS.$(this).remove();
            apply();
        });
    }

    function deleteBodyCleanupRegexp(item) {
        var regexpId = AJS.$(item).attr('href');
        //console.log('deleteBodyCleanupRegexp() : '+item);
        AJS.$('tr[id=bodycleanupregexp_' + regexpId + ']').hide('fast', function () {
            AJS.$(this).remove();
            apply();
        });
    }

    function deleteDelimRegexp(item) {
        var regexpId = AJS.$(item).attr('href');
        //console.log('deleteDelimRegexp() : '+item);
        AJS.$('tr[id=bodydelimregexp_' + regexpId + ']').hide('fast', function () {
            AJS.$(this).remove();
            apply();
        });
    }

    function hookDeleteCleanupRegexpItem() {
        AJS.$(".deleteSubjectCleanupRegexpLink").click(function (e) {
            //console.log('click on .deleteSubjectCleanupRegexpLink');
            e.preventDefault();
            deleteSubjectCleanupRegexp(AJS.$(this));
            return false;
        });

        AJS.$(".deleteBodyCleanupRegexpLink").click(function (e) {
            //console.log('click on .deleteBodyCleanupRegexpLink');
            e.preventDefault();
            deleteBodyCleanupRegexp(AJS.$(this));
            return false;
        });
    }

    function hookDeleteDelimRegexpItem() {
        AJS.$(".deleteDelimRegexpLink").click(function (e) {
            e.preventDefault();
            deleteDelimRegexp(AJS.$(this));
            return false;
        });
    }

    function hookEvalSubmit() {
        AJS.$("#dynEvalApplyButton").click(function (e) {
            e.preventDefault();

            apply();

            return false;
        });
    }

    function hookAutoEval() {
        AJS.$('.jemhAutoEvalFocusMember').blur(function () {
            //console.log('blur');
            apply();
        });
    }

    function hookTestCaseSelect() {
        AJS.$('#dynEvalTestCaseSelect').change(function (e) {
            if (AJS.$(this).val() == "") {
                //console.log('disabling manual eval');
                AJS.$('#dynEvalApplyButton').attr('disabled', 'disabled');
                AJS.$('#dynEvalRawDivOutput').html('');
                AJS.$('#dynEvalModifiedDivOutput').html('');
            } else {
                //console.log('enabling manual eval');
                AJS.$('#dynEvalApplyButton').removeAttr('disabled');
                apply();
            }
        });
    }

    function hookBodyDelimiterCreateCommentChange() {
        AJS.$('.jemhBodyDelimCheckbox').change(function (e) {
            apply();
        });
    }

    function hookEvalModeChange() {
        AJS.$('#dynEvalModeSelect').change(function (e) {
            apply();
        });
    }

    function hookProjectMappingSubmit() {
        //console.log('hookProjectMappingSumit()');

        AJS.$('#jemhProjectConfigForm').submit(function (eventObj) {

            var formObj = AJS.$(this);

            //console.log('*** injecting cleanup regexps into the form');

            //SUBJECT CLEANUP REGEXPS
            AJS.$(".jemhSubjectCleanupRegexp").each(function (i, obj) {
                var obj = AJS.$(obj);
                if (obj.val().trim() != "") {
                    var cleanupType = "subject";
                    var objId = obj.attr('id').split('_')[1];
                    //console.log('has cleanupId : '+objId);
                    var tgtVal = "" + objId + "," + cleanupType + "|" + obj.val();

                    //console.log('cleanupTgtVal: '+tgtVal);

                    //console.log('found cleanupRegexps #'+i+' id='+objId+', cleanupType='+cleanupType+' value='+obj.val());
                    var escaped = encodeRFC5987ValueChars2(AJS.$('<div></div>').html(tgtVal).text());
                    //console.log('escaped subject cleanup value: '+escaped);

                    formObj.append("<input type='hidden' name='cleanupRegexps' value='" + escaped + "'>");
                } else {
                    //console.log('ignoring empty jemhCleanupSubjectRegexp');
                }
            });

            //BODY CLEANUP REGEXPS
            AJS.$(".jemhBodyCleanupRegexp").each(function (i, obj) {
                var obj = AJS.$(obj);
                if (obj.val().trim() != "") {
                    var cleanupType = "body";
                    var objId = obj.attr('id').split('_')[1];
                    //console.log('has cleanupId : '+objId);
                    var tgtVal = "" + objId + "," + cleanupType + "|" + obj.val();

                    //console.log('cleanupTgtVal: '+tgtVal);

                    //console.log('found cleanupRegexps #'+i+' id='+objId+', cleanupType='+cleanupType+' value='+obj.val());

                    var escaped = encodeRFC5987ValueChars2(AJS.$('<div></div>').html(tgtVal).text());
                    //console.log('escaped body cleanup value: '+escaped);

                    formObj.append("<input type='hidden' name='cleanupRegexps' value='" + escaped + "'>");
                } else {
                    //console.log('ignoring empty jemhCleanupBodyRegexp');
                }
            });

            //console.log('*** injecting delimiter regexps into the form');

            //BODY DELIMS
            AJS.$(".jemhDelimiterRegexp").each(function (i, obj) {
                var obj = AJS.$(obj);
                if (obj.val().trim() != "") {
                    var objId = obj.attr('id').split('_')[1];
                    //console.log('has delimId : '+objId);

                    var useOnCreate = AJS.$('#on_create_' + objId).attr('checked') == 'checked';
                    var useOnComment = AJS.$('#on_comment_' + objId).attr('checked') == 'checked';

                    var tgtVal = objId + "," + useOnCreate + "," + useOnComment + "|" + obj.val();

                    //console.log('delimTgtVal: '+tgtVal);

                    //console.log('found delimiterRegexps #'+i+', value is : '+obj.val());

                    var escaped = encodeRFC5987ValueChars2(AJS.$('<div></div>').html(tgtVal).text());

                    //console.log('escaped delim value: '+escaped);
                    formObj.append("<input type='hidden' name='delimiterRegexps' value='" + escaped + "'>");
                } else {
                    //console.log('ignoring empty jemhDelimiterRegexp');
                }
            });

            //console.log('*** DONE appending some inputs to the form');
            return true;
        });
    }

    function init() {
        AJS.log('pmdelimregexp.js loaded');
        hookAutoEval();
        hookDeleteDelimRegexpItem();
        hookDeleteCleanupRegexpItem();
        hookAddSubjectCleanupItem();
        hookAddBodyCleanupItem();
        hookAddRegexpDelimItem();
        hookBodyDelimiterCreateCommentChange();
        hookEvalModeChange();
        hookTestCaseSelect();
        hookEvalSubmit();
        hookProjectMappingSubmit();
    }

    init();
});