AJS.$(document).ready(function () {
	console.log('scriptfp.js loaded');

	function htmlEncode(value) {
		// Create a in-memory div, set its inner text (which jQuery automatically encodes)
		// Then grab the encoded contents back out. The div never exists on the page.
		return AJS.$('<div/>').text(value).html();
	}

	function runScript(editor, editWrapperDivId, consoleLogDiv, mapDiv) {
		AJS.$("#ruleMatchButton").attr('disabled', true);
		AJS.$("#evalInfo").hide();
		AJS.$("#evalOutcome").attr('class', 'aui-lozenge aui-lozenge-subtle');
		AJS.$("#evalOutcomeReason").empty();

		AJS.$(consoleLogDiv).text("Running, please wait...");
		//AJAX POST of content to render

		let languageEngine = AJS.$("#jemhScriptFieldProcForm [name=scriptLanguage]").val();
		let currentLanguage =languageEngine.split("_")[0];
		let currentEngine = "";
		if (languageEngine.length > 1) {
			currentEngine =languageEngine.split("_")[1];
		}
		console.log('currentLanguage: '+currentLanguage);
		console.log('currentEngine: '+currentEngine);

		var currentScript = editor.getSession().getValue();
		var currentTestCaseId = AJS.$("#testCasePicker").val();

		AJS.$("#jemhScriptDataForm [name=testCaseId]").val(currentTestCaseId);
		AJS.$("#jemhScriptDataForm [name=scriptContent]").val(currentScript);		
		AJS.$("#jemhScriptDataForm [name=scriptLanguage]").val(currentLanguage);
		AJS.$("#jemhScriptDataForm [name=scriptEngine]").val(currentEngine);

		var postData = AJS.$('#jemhScriptDataForm').serialize();
		var formUrl = AJS.$('#jemhScriptDataForm').attr('action');

		AJS.$(consoleLogDiv).css({border: "3px solid #CCCCCC"});
		AJS.$(consoleLogDiv).html("<h3>Running, please wait...</h3>");

		//contentType: sent to the server as a header
		//dataType is the response
		var jqXHR = AJS.$.ajax({
			type: "POST",
			url: formUrl,
			dataType: "json",
			data: postData,
			timeout: 60000
		}); //post

		jqXHR.done(function (data, textStatus, errorThrown) {
			AJS.$("#evalInfo").show();
	  		AJS.$("#mailProcessingDuration > div").text(data.duration+" ms [engineInit="+data.scriptEngineInitTime
	  			+"ms,  scriptCompile="+data.scriptCompileTime
	  			+"ms, scriptEval="+data.scriptEngineEvalTime+"ms]").html();			

			if (data.exception) {
				AJS.$(consoleLogDiv).css({border: "3px solid #DE350B"});
				AJS.$(consoleLogDiv).text(data.exception);
				AJS.$("#evalOutcome").text("Exception").addClass("aui-lozenge-error");
			} else {
				if (data.outcome) {
					AJS.$(consoleLogDiv).css({border: "3px solid #FF991F"});
					AJS.$("#evalOutcome").text(data.outcome).addClass("aui-lozenge-moved");
					AJS.$("#evalOutcomeReason").text(data.outcomeReason);
				} else {
					AJS.$(consoleLogDiv).css({border: "3px solid #00875A"});
					AJS.$("#evalOutcome").text("OK").addClass("aui-lozenge-success");
				}

				if (data.consoleLog) {
					console.log('consoleLog: ' + data.consoleLog);
					AJS.$(consoleLogDiv).text(data.consoleLog);
				} else {
					AJS.$(consoleLogDiv).text("Script returned no console output");
				}

				AJS.$('#scriptMapResultTable tbody').find("tr").remove();

				for (var key in data.map) {
					var attrName = key;
					var attrValue = data.map[key];
					console.log('mapped ' + attrName + ' = ' + attrValue);
					var row = '<tr><th>' + htmlEncode(attrName) + '</th><td>' + htmlEncode(attrValue) + '</td></tr>';
					AJS.$('#scriptMapResultTable tbody').append(row);
				}
			}
		})
			.fail(function (data, textStatus, errorMessage) {
				console.error("Script execution failed: textStatus=" + textStatus + ", errorMessage: " + errorMessage);
				AJS.$(consoleLogDiv).text('');
				AJS.messages.generic(consoleLogDiv, {
					title: "Script rendering failed",
					body: '<p>Status: ' + textStatus + ', error: ' + errorMessage + '</p>'
				});
			})
			.always(function () {
				AJS.$("#ruleMatchButton").removeAttr('disabled');
			});
	}

	var scriptEditor = buildAceEditor('jemhScriptFieldProcForm', 'editor0', 'statusBar', 'scriptContent', 'javascript');

	AJS.$("#ruleMatchButton").click(function (e) {
		e.preventDefault();
		runScript(scriptEditor, "#editor0", "#scriptfpConsole", "#scriptfpMap");
		return false;
	}); //click

	AJS.$(function () {
		var toggle = new JIRA.ToggleBlock({blockSelector: ".toggle-wrap", triggerSelector: ".mod-header h3"})
	});

	//Script Language
	AJS.$(".jemhScriptFPLanguage").change(function(){
		let languageValEngineVal = AJS.$(".jemhScriptFPLanguage").find(":selected").val();
		let engineVal = "";
		let languageVal = languageValEngineVal.split("_")[0];
		if (languageValEngineVal.length > 1) {
			engineVal = languageValEngineVal.split("_")[1];
		}
		AJS.$("#scriptLanguage").text(languageVal);
		AJS.$("#scriptEngine").text(engineVal);
		checkEngineAndLanguageValidation(languageVal, engineVal);
	});
	let languageValEngineVal = AJS.$(".jemhScriptFPLanguage").find(":selected").val();
	let engineVal = "";
	let languageVal = languageValEngineVal.split("_")[0];
	if (languageValEngineVal.length > 1) {
		engineVal = languageValEngineVal.split("_")[1];
	}
	checkEngineAndLanguageValidation(languageVal, engineVal);
	loadScriptContext('scriptfp');
});

function checkEngineAndLanguageValidation(languageVal, engineVal) {
	//Is Language compatible
	if(engineVal !== "" && languageVal !== ""){
		AJS.$('#scriptFPLanguageStatus').hide();
		AJS.$(".submit").removeAttr("disabled");
		AJS.$("#ruleMatchButton").removeAttr("disabled");
		AJS.$(".submit").removeAttr("disabled");
	}
	if (!engineVal) {
		AJS.$('#scriptFPLanguageStatus').show();
		AJS.$("#ruleMatchButton").attr("disabled", "disabled");
		AJS.$(".submit").attr("disabled", "disabled");
	}
	if (!languageVal) {
		AJS.$('#scriptFPLanguageStatus').show();		
		AJS.$("#ruleMatchButton").attr("disabled", "disabled");
		AJS.$(".submit").attr("disabled", "disabled");
	}
}
